@@ -22,7 +22,7 @@ use crate::device::DiskDevices;
 use crate::file::{FileId, FileLen, FileMetadata};
 use crate::group::{FileGroup, FileSubGroup};
 use crate::lock::FileLock;
-use crate::log::Log;
+use crate::log::{Log, LogExt};
 use crate::path::Path;
 use crate::util::{max_result, min_result, try_sort_by_key};
 use crate::{Error, TIMESTAMP_FMT};
@@ -265,8 +265,8 @@ impl FsCommand {
     /// If the function succeeds, removes the file permanently.
     pub fn safe_remove<R>(
         path: &Path,
-        f: impl Fn(&Path) -> io::Result<R>,
-        log: &Log,
+        f: impl FnOnce(&Path) -> io::Result<R>,
+        log: &dyn Log,
     ) -> io::Result<R> {
         let tmp = Self::temp_file(path);
         Self::unsafe_rename(path, &tmp)?;
@@ -297,7 +297,7 @@ impl FsCommand {
     }
 
     /// Executes the command and returns the number of bytes reclaimed
-    pub fn execute(&self, should_lock: bool, log: &Log) -> io::Result<FileLen> {
+    pub fn execute(&self, should_lock: bool, log: &dyn Log) -> io::Result<FileLen> {
         match self {
             FsCommand::Remove { file } => {
                 let _ = Self::maybe_lock(&file.path, should_lock)?;
@@ -471,7 +471,7 @@ impl AddAssign for DedupeResult {
 
 /// Returns true if any of the files have been modified after the given timestamp.
 /// Also returns true if file timestamp could not be read.
-fn was_modified(files: &[PathAndMetadata], after: DateTime<FixedOffset>, log: &Log) -> bool {
+fn was_modified(files: &[PathAndMetadata], after: DateTime<FixedOffset>, log: &dyn Log) -> bool {
     let mut result = false;
     let after: DateTime<Local> = after.into();
     for PathAndMetadata {
@@ -728,7 +728,7 @@ impl PartitionedFileGroup {
 
 /// Attempts to retrieve the metadata of all the files in the file group.
 /// If metadata is inaccessible for a file, a warning is emitted to the log, and None gets returned.
-fn files_metadata<P>(group: FileGroup<P>, log: &Log) -> Option<Vec<PathAndMetadata>>
+fn files_metadata<P>(group: FileGroup<P>, log: &dyn Log) -> Option<Vec<PathAndMetadata>>
 where
     P: Into<Path>,
 {
@@ -757,13 +757,13 @@ where
 fn partition<P>(
     group: FileGroup<P>,
     config: &DedupeConfig,
-    log: &Log,
+    log: &dyn Log,
 ) -> Result<PartitionedFileGroup, Error>
 where
     P: Into<Path>,
 {
     let file_len = group.file_len;
-    let file_hash = group.file_hash;
+    let file_hash = group.file_hash.clone();
     let error = |msg: &str| {
         Err(Error::from(format!(
             "Could not determine files to drop in group with hash {} and len {}: {}",
@@ -881,7 +881,7 @@ pub fn dedupe<'a, I, P>(
     groups: I,
     op: DedupeOp,
     config: &'a DedupeConfig,
-    log: &'a Log,
+    log: &'a dyn Log,
 ) -> impl ParallelIterator<Item = (usize, Vec<FsCommand>)> + Send + 'a
 where
     I: IntoIterator<Item = FileGroup<P>> + 'a,
@@ -910,7 +910,7 @@ where
 /// On command execution failure, a warning is logged and the execution of remaining commands
 /// continues.
 /// Returns the number of files processed and the amount of disk space reclaimed.
-pub fn run_script<I>(script: I, should_lock: bool, log: &Log) -> DedupeResult
+pub fn run_script<I>(script: I, should_lock: bool, log: &dyn Log) -> DedupeResult
 where
     I: IntoParallelIterator<Item = (usize, Vec<FsCommand>)>,
 {
@@ -1035,6 +1035,7 @@ mod test {
     use std::collections::HashSet;
     use std::fs::{create_dir, create_dir_all};
     use std::path::PathBuf;
+    use std::str::FromStr;
     use std::{thread, time};
 
     use chrono::Duration;
@@ -1043,6 +1044,7 @@ mod test {
     use crate::config::GroupConfig;
     use crate::file::FileHash;
     use crate::group_files;
+    use crate::log::StdLog;
     use crate::pattern::Pattern;
     use crate::util::test::{create_file, create_file_newer_than, read_file, with_dir, write_file};
 
@@ -1063,7 +1065,7 @@ mod test {
     #[test]
     fn test_remove_command_removes_file() {
         with_dir("dedupe/remove_cmd", |root| {
-            let log = Log::new();
+            let log = StdLog::new();
             let file_path = root.join("file");
             create_file(&file_path);
             let file = PathAndMetadata::new(Path::from(&file_path)).unwrap();
@@ -1076,7 +1078,7 @@ mod test {
     #[test]
     fn test_move_command_moves_file_by_rename() {
         with_dir("dedupe/move_rename_cmd", |root| {
-            let log = Log::new();
+            let log = StdLog::new();
             let file_path = root.join("file");
             let target = Path::from(root.join("target"));
             create_file(&file_path);
@@ -1095,7 +1097,7 @@ mod test {
     #[test]
     fn test_move_command_moves_file_by_copy() {
         with_dir("dedupe/move_copy_cmd", |root| {
-            let log = Log::new();
+            let log = StdLog::new();
             let file_path = root.join("file");
             let target = Path::from(root.join("target"));
             create_file(&file_path);
@@ -1114,7 +1116,7 @@ mod test {
     #[test]
     fn test_move_fails_if_target_exists() {
         with_dir("dedupe/move_target_exists", |root| {
-            let log = Log::new();
+            let log = StdLog::new();
             let file_path = root.join("file");
             let target = root.join("target");
             create_file(&file_path);
@@ -1132,7 +1134,7 @@ mod test {
     #[test]
     fn test_soft_link_command_replaces_file_with_a_link() {
         with_dir("dedupe/soft_link_cmd", |root| {
-            let log = Log::new();
+            let log = StdLog::new();
             let file_path_1 = root.join("file_1");
             let file_path_2 = root.join("file_2");
             write_file(&file_path_1, "foo");
@@ -1159,7 +1161,7 @@ mod test {
     #[test]
     fn test_hard_link_command_replaces_file_with_a_link() {
         with_dir("dedupe/hard_link_cmd", |root| {
-            let log = Log::new();
+            let log = StdLog::new();
             let file_path_1 = root.join("file_1");
             let file_path_2 = root.join("file_2");
 
@@ -1206,9 +1208,9 @@ mod test {
     #[test]
     fn test_partition_selects_files_for_removal() {
         with_dir("dedupe/partition/basic", |root| {
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
             let config = DedupeConfig::default();
-            let partitioned = partition(group, &config, &Log::new()).unwrap();
+            let partitioned = partition(group, &config, &StdLog::new()).unwrap();
             assert_eq!(partitioned.to_keep.len(), 1);
             assert_eq!(partitioned.to_drop.len(), 2);
         })
@@ -1217,10 +1219,10 @@ mod test {
     #[test]
     fn test_partition_bails_out_if_file_modified_too_late() {
         with_dir("dedupe/partition/modification", |root| {
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
             let mut config = DedupeConfig::default();
             config.modified_before = Some(DateTime::from(Local::now() - Duration::days(1)));
-            let partitioned = partition(group, &config, &Log::new());
+            let partitioned = partition(group, &config, &StdLog::new());
             assert!(partitioned.is_err());
         })
     }
@@ -1228,13 +1230,13 @@ mod test {
     #[test]
     fn test_partition_skips_file_with_different_len() {
         with_dir("dedupe/partition/file_len", |root| {
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
             let path = group.files[0].clone();
             write_file(&path.to_path_buf(), "foo");
 
             let mut config = DedupeConfig::default();
             config.priority = vec![Priority::MostRecentlyModified];
-            let partitioned = partition(group, &config, &Log::new()).unwrap();
+            let partitioned = partition(group, &config, &StdLog::new()).unwrap();
             assert!(partitioned
                 .to_drop
                 .iter()
@@ -1260,12 +1262,12 @@ mod test {
                 // file creation time
                 return;
             }
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
             let mut config = DedupeConfig::default();
             config.priority = vec![Priority::Newest];
-            let partitioned_1 = partition(group.clone(), &config, &Log::new()).unwrap();
+            let partitioned_1 = partition(group.clone(), &config, &StdLog::new()).unwrap();
             config.priority = vec![Priority::Oldest];
-            let partitioned_2 = partition(group.clone(), &config, &Log::new()).unwrap();
+            let partitioned_2 = partition(group.clone(), &config, &StdLog::new()).unwrap();
 
             assert_ne!(
                 path_set(&partitioned_1.to_keep),
@@ -1281,7 +1283,7 @@ mod test {
     #[test]
     fn test_partition_respects_modification_time_priority() {
         with_dir("dedupe/partition/mtime_priority", |root| {
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
 
             thread::sleep(time::Duration::from_millis(10));
             let path = group.files[0].clone();
@@ -1289,9 +1291,9 @@ mod test {
 
             let mut config = DedupeConfig::default();
             config.priority = vec![Priority::MostRecentlyModified];
-            let partitioned_1 = partition(group.clone(), &config, &Log::new()).unwrap();
+            let partitioned_1 = partition(group.clone(), &config, &StdLog::new()).unwrap();
             config.priority = vec![Priority::LeastRecentlyModified];
-            let partitioned_2 = partition(group.clone(), &config, &Log::new()).unwrap();
+            let partitioned_2 = partition(group.clone(), &config, &StdLog::new()).unwrap();
 
             assert_ne!(
                 path_set(&partitioned_1.to_keep),
@@ -1307,17 +1309,17 @@ mod test {
     #[test]
     fn test_partition_respects_keep_patterns() {
         with_dir("dedupe/partition/keep", |root| {
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
             let mut config = DedupeConfig::default();
             config.priority = vec![Priority::LeastRecentlyModified];
             config.keep_name_patterns = vec![Pattern::glob("*_1").unwrap()];
-            let p = partition(group.clone(), &config, &Log::new()).unwrap();
+            let p = partition(group.clone(), &config, &StdLog::new()).unwrap();
             assert_eq!(p.to_keep.len(), 1);
             assert_eq!(&p.to_keep[0].path, &group.files[0]);
 
             config.keep_name_patterns = vec![];
             config.keep_path_patterns = vec![Pattern::glob("**/file_1").unwrap()];
-            let p = partition(group.clone(), &config, &Log::new()).unwrap();
+            let p = partition(group.clone(), &config, &StdLog::new()).unwrap();
             assert_eq!(p.to_keep.len(), 1);
             assert_eq!(&p.to_keep[0].path, &group.files[0]);
         })
@@ -1326,17 +1328,17 @@ mod test {
     #[test]
     fn test_partition_respects_drop_patterns() {
         with_dir("dedupe/partition/drop", |root| {
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
             let mut config = DedupeConfig::default();
             config.priority = vec![Priority::LeastRecentlyModified];
             config.name_patterns = vec![Pattern::glob("*_3").unwrap()];
-            let p = partition(group.clone(), &config, &Log::new()).unwrap();
+            let p = partition(group.clone(), &config, &StdLog::new()).unwrap();
             assert_eq!(p.to_drop.len(), 1);
             assert_eq!(&p.to_drop[0].path, &group.files[2]);
 
             config.name_patterns = vec![];
             config.path_patterns = vec![Pattern::glob("**/file_3").unwrap()];
-            let p = partition(group.clone(), &config, &Log::new()).unwrap();
+            let p = partition(group.clone(), &config, &StdLog::new()).unwrap();
             assert_eq!(p.to_drop.len(), 1);
             assert_eq!(&p.to_drop[0].path, &group.files[2]);
         })
@@ -1350,8 +1352,8 @@ mod test {
             create_dir(&root1).unwrap();
             create_dir(&root2).unwrap();
 
-            let group1 = make_group(&root1, FileHash(0));
-            let group2 = make_group(&root2, FileHash(0));
+            let group1 = make_group(&root1, FileHash::from_str("00").unwrap());
+            let group2 = make_group(&root2, FileHash::from_str("00").unwrap());
             let group = FileGroup {
                 file_len: group1.file_len,
                 file_hash: group1.file_hash,
@@ -1361,7 +1363,7 @@ mod test {
             let mut config = DedupeConfig::default();
             config.isolated_roots = vec![Path::from(&root1), Path::from(&root2)];
 
-            let p = partition(group.clone(), &config, &Log::new()).unwrap();
+            let p = partition(group.clone(), &config, &StdLog::new()).unwrap();
             assert_eq!(p.to_drop.len(), 3);
             assert!(p
                 .to_drop
@@ -1395,7 +1397,7 @@ mod test {
 
             let group = FileGroup {
                 file_len: FileLen(3),
-                file_hash: FileHash(0),
+                file_hash: FileHash::from_str("00").unwrap(),
                 files: vec![
                     Path::from(&file_b1),
                     Path::from(&file_a2),
@@ -1405,7 +1407,7 @@ mod test {
             };
 
             let config = DedupeConfig::default();
-            let p = partition(group.clone(), &config, &Log::new()).unwrap();
+            let p = partition(group.clone(), &config, &StdLog::new()).unwrap();
 
             // drop A files because file_a2 appears after file_b1 in the files vector
             assert_eq!(p.to_drop.len(), 2);
@@ -1424,11 +1426,11 @@ mod test {
     #[test]
     fn test_run_dedupe_script() {
         with_dir("dedupe/partition/run_dedupe_script", |root| {
-            let mut log = Log::new();
+            let mut log = StdLog::new();
             log.no_progress = true;
             log.log_stderr_to_stdout = true;
 
-            let group = make_group(root, FileHash(0));
+            let group = make_group(root, FileHash::from_str("00").unwrap());
             let mut config = DedupeConfig::default();
             config.priority = vec![Priority::LeastRecentlyModified];
             let script = dedupe(vec![group], DedupeOp::Remove, &config, &log);
@@ -1443,13 +1445,13 @@ mod test {
     #[test]
     fn test_log_dedupe_script() {
         with_dir("dedupe/partition/log_dedupe_script", |root| {
-            let mut log = Log::new();
+            let mut log = StdLog::new();
             log.no_progress = true;
             log.log_stderr_to_stdout = true;
 
-            let group_1 = make_group(&root.join("group_1"), FileHash(0));
-            let group_2 = make_group(&root.join("group_2"), FileHash(1));
-            let group_3 = make_group(&root.join("group_3"), FileHash(2));
+            let group_1 = make_group(&root.join("group_1"), FileHash::from_str("00").unwrap());
+            let group_2 = make_group(&root.join("group_2"), FileHash::from_str("01").unwrap());
+            let group_3 = make_group(&root.join("group_3"), FileHash::from_str("02").unwrap());
             let groups = vec![group_1, group_2, group_3];
 
             let mut config = DedupeConfig::default();
@@ -1476,7 +1478,7 @@ mod test {
     #[test]
     fn test_hard_link_merges_subgroups_of_hard_links() {
         with_dir("dedupe/merge_subgroups_of_hardlinks", |root| {
-            let mut log = Log::new();
+            let mut log = StdLog::new();
             log.no_progress = true;
             log.log_stderr_to_stdout = true;
 
@@ -1520,7 +1522,7 @@ mod test {
         use std::os::unix::fs;
 
         with_dir("dedupe/remove_subgroups_with_symlinks", |root| {
-            let mut log = Log::new();
+            let mut log = StdLog::new();
             log.no_progress = true;
             log.log_stderr_to_stdout = true;
 
