@@ -444,8 +444,8 @@ mod test {
     use std::io::Write;
 
     use crate::file::{FileChunk, FileLen, FilePos};
-    use crate::hasher::FileHasher;
-    use crate::log::Log;
+    use crate::hasher::{FileHasher, HashFn};
+    use crate::log::StdLog;
     use crate::util::test::with_dir;
 
     use super::*;
@@ -466,8 +466,8 @@ mod test {
             input.write(content).unwrap();
             drop(input);
 
-            let log = Log::default();
-            let hasher = FileHasher::new(Some(transform), &log);
+            let log = StdLog::default();
+            let hasher = FileHasher::new(HashFn::default(), Some(transform), &log);
             let input_path = Path::from(input_path);
             let chunk = FileChunk::new(&input_path, FilePos(0), FileLen::MAX);
             let good_file_hash = hasher.hash_file(&chunk, |_| {}).unwrap();
@@ -488,8 +488,8 @@ mod test {
             input.write(content).unwrap();
             drop(input);
 
-            let log = Log::default();
-            let hasher = FileHasher::new(Some(transform), &log);
+            let log = StdLog::default();
+            let hasher = FileHasher::new(HashFn::default(), Some(transform), &log);
             let input_path = Path::from(input_path);
 
             let chunk = FileChunk::new(&input_path, FilePos(0), FileLen::MAX);
